<?php
/*

Copyright 2022 Coin Payment.

*/

class CoinPayment_CoinPayment_PaymentController extends Mage_Core_Controller_Front_Action
{
    // The redirect action is triggered when someone places an order
    public function redirectAction()
    {
        $this->loadLayout();
        $block = $this->getLayout()->createBlock('Mage_Core_Block_Template', 'coinpayment', array('template' => 'coinpayment/redirect.phtml'));
        $this->getLayout()->getBlock('content')->append($block);
        $this->renderLayout();
    }

    private function _errorAndDie($error_msg)
    {
        $email = Mage::getStoreConfig('payment/coinpayment/ipn_debug');
        if (!empty($email)) {
            $report = "Error Message: " . $error_msg . "\n\n";

            $report .= "AUTH User: " . $_SERVER['PHP_AUTH_USER'] . "\n";
            $report .= "AUTH Pass: " . $_SERVER['PHP_AUTH_PW'] . "\n\n";

            $report .= "POST Fields\n\n";
            foreach ($_POST as $key => $value) {
                $report .= $key . '=' . html_entity_decode($value, ENT_QUOTES, 'UTF-8') . "\n";
            }

            @mail($email, "Coin Payments Invalid IPN", $report);
        }
        die('IPN Error: ' . $error_msg);
    }

    function _is_ipn_valid($ipn, $order = null)
    {
        if (!isset($ipn['ipn_mode'])) {
            $this->_errorAndDie('IPN received with no ipn_mode.');
        }
        if ($ipn['ipn_mode'] == 'hmac') {
            if (!isset($_SERVER['HTTP_HMAC']) || empty($_SERVER['HTTP_HMAC'])) {
                $this->_errorAndDie('No HMAC signature sent.');
            }

            $request = file_get_contents('php://input');
            if ($request === FALSE || empty($request)) {
                $this->_errorAndDie('Error reading POST data: ' . print_r($_SERVER, TRUE) . '/' . print_r($_POST, TRUE));
            }

            $merchant = isset($ipn['merchant']) ? $ipn['merchant'] : '';
            if (empty($merchant)) {
                $this->_errorAndDie('No Merchant ID passed');
            }
            if ($merchant != trim(Mage::getStoreConfig('payment/coinpayment/merchant_id'))) {
                $this->_errorAndDie('Invalid Merchant ID');
            }

            return TRUE;
        } else if ($ipn['ipn_mode'] == 'httpauth') {
            $merchant = isset($ipn['merchant']) ? $ipn['merchant'] : '';
            if (empty($merchant)) {
                $this->_errorAndDie('No Merchant ID passed');
            }
            if ($merchant != trim(Mage::getStoreConfig('payment/coinpayment/merchant_id'))) {
                $this->_errorAndDie('Invalid Merchant ID');
            }
            return TRUE;
        } else {
            $this->_errorAndDie('Unknown ipn_mode.');
        }
        return FALSE;
    }

    // The response action is triggered when your gateway sends back a response after processing the customer's payment
    public function responseAction()
    {
        if ($this->getRequest()->isPost()) {
            if ($this->_is_ipn_valid($_POST)) {
                // Payment was successful, so update the order's state, send order email and move to the success page
                $order_id = intval($_POST['invoice']);
                $order = Mage::getModel('sales/order');
                if ($order->loadByIncrementId($order_id)) {
                    if ($order->getState() == Mage_Sales_Model_Order::STATE_NEW) {
                        if ($_POST['ipn_type'] == 'button') {
                            if ($_POST['currency'] == $order->getBaseCurrency()->getCode()) {
                                if ($_POST['amount'] >= $order->getBaseGrandTotal()) {
                                    $status = intval($_POST['status']);
                                    if ($status < 0) {
                                        //canceled or timed out
                                        $order->cancel();
                                        $order->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Coin Payment Status: ' . $_POST['status_text']);
                                    } else if ($status >= 100 || $status == 2) {
                                        //order complete or queued for nightly payout
                                        $str = 'Payment Status: ' . $_POST['status_text'] . '<br />';
                                        $str .= 'Transaction ID: ' . $_POST['txn_id'] . '<br />';
                                        $str .= 'Original Amount: ' . sprintf('%.08f', $_POST['amount']) . '<br />';
                                        $str .= 'Received Amount: ' . sprintf('%.08f', $_POST['amount']);
                                        $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true, $str);
                                        $order->sendNewOrderEmail();
                                        $order->setEmailSent(true);
                                    } else {
                                        //order pending
                                        $order->setState(Mage_Sales_Model_Order::STATE_NEW, true, 'Payment Status: ' . $_POST['status_text']);
                                    }
                                    $order->save();
                                    die('IPN OK');
                                } else {
                                    $this->_errorAndDie('Amount paid is less than order total!', $order);
                                }
                            } else {
                                $this->_errorAndDie('Original currency does not match!', $order);
                            }
                        } else {
                            $this->_errorAndDie('Invalid IPN type!', $order);
                        }
                    } else {
                        $this->_errorAndDie('Order is no longer new. (most likely IPN has already been processed)');
                    }
                } else {
                    $this->_errorAndDie('Could not load order with ID: ' . $order_id);
                }
            }
        } else {
            $this->_errorAndDie('Request is not POST');
        }
    }

    // The cancel action is triggered when an order is to be cancelled
    public function cancelAction()
    {
        if (Mage::getSingleton('checkout/session')->getLastRealOrderId()) {
            $order = Mage::getModel('sales/order')->loadByIncrementId(Mage::getSingleton('checkout/session')->getLastRealOrderId());
            if ($order->getId()) {
                // Flag the order as 'cancelled' and save it
                $order->cancel()->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Order canceled')->save();
            }
        }
    }
}